// This software is released into the Public Domain.  See copying.txt for details.
package org.openstreetmap.osmosis.core.plugin;

import org.java.plugin.Plugin;


/**
 * The core plugin entry point.
 *
 * @author Marcus Wolschon
 */
public class CorePlugin extends Plugin {

	/**
	 * {@inheritDoc}
	 */
	@Override
	protected void doStart() throws Exception {
		// ignored
	}


	/**
	 * {@inheritDoc}
	 */
	@Override
	protected void doStop() throws Exception {
		// ignored
	}
}
