// This software is released into the Public Domain.  See copying.txt for details.
package org.openstreetmap.osmosis.core.task.common;


/**
 * Defines the root of the interface inheritance hierarchy for all task types.
 * 
 * @author Brett Henderson
 */
public interface Task {
	// Interface only exists to provide a common hierarchy root for all task
	// types.
}
